# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.delegated_checkout._order_retrieve_params import (
        OrderRetrieveParams as OrderRetrieveParams,
    )
    from stripe.params.delegated_checkout._requested_session_confirm_params import (
        RequestedSessionConfirmParams as RequestedSessionConfirmParams,
        RequestedSessionConfirmParamsAffiliateAttribution as RequestedSessionConfirmParamsAffiliateAttribution,
        RequestedSessionConfirmParamsAffiliateAttributionSource as RequestedSessionConfirmParamsAffiliateAttributionSource,
        RequestedSessionConfirmParamsBuyerConsents as RequestedSessionConfirmParamsBuyerConsents,
        RequestedSessionConfirmParamsBuyerConsentsMarketing as RequestedSessionConfirmParamsBuyerConsentsMarketing,
        RequestedSessionConfirmParamsBuyerConsentsMarketingConsent as RequestedSessionConfirmParamsBuyerConsentsMarketingConsent,
        RequestedSessionConfirmParamsRiskDetails as RequestedSessionConfirmParamsRiskDetails,
        RequestedSessionConfirmParamsRiskDetailsClientDeviceMetadataDetails as RequestedSessionConfirmParamsRiskDetailsClientDeviceMetadataDetails,
    )
    from stripe.params.delegated_checkout._requested_session_create_params import (
        RequestedSessionCreateParams as RequestedSessionCreateParams,
        RequestedSessionCreateParamsAffiliateAttribution as RequestedSessionCreateParamsAffiliateAttribution,
        RequestedSessionCreateParamsAffiliateAttributionSource as RequestedSessionCreateParamsAffiliateAttributionSource,
        RequestedSessionCreateParamsDiscounts as RequestedSessionCreateParamsDiscounts,
        RequestedSessionCreateParamsFulfillmentDetails as RequestedSessionCreateParamsFulfillmentDetails,
        RequestedSessionCreateParamsFulfillmentDetailsAddress as RequestedSessionCreateParamsFulfillmentDetailsAddress,
        RequestedSessionCreateParamsLineItemDetail as RequestedSessionCreateParamsLineItemDetail,
        RequestedSessionCreateParamsPaymentMethodOptions as RequestedSessionCreateParamsPaymentMethodOptions,
        RequestedSessionCreateParamsPaymentMethodOptionsCard as RequestedSessionCreateParamsPaymentMethodOptionsCard,
        RequestedSessionCreateParamsSellerDetails as RequestedSessionCreateParamsSellerDetails,
    )
    from stripe.params.delegated_checkout._requested_session_expire_params import (
        RequestedSessionExpireParams as RequestedSessionExpireParams,
    )
    from stripe.params.delegated_checkout._requested_session_list_orders_params import (
        RequestedSessionListOrdersParams as RequestedSessionListOrdersParams,
    )
    from stripe.params.delegated_checkout._requested_session_modify_params import (
        RequestedSessionModifyParams as RequestedSessionModifyParams,
        RequestedSessionModifyParamsDiscounts as RequestedSessionModifyParamsDiscounts,
        RequestedSessionModifyParamsFulfillmentDetails as RequestedSessionModifyParamsFulfillmentDetails,
        RequestedSessionModifyParamsFulfillmentDetailsAddress as RequestedSessionModifyParamsFulfillmentDetailsAddress,
        RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOption as RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOption,
        RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionDigital as RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionDigital,
        RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverride as RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverride,
        RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideDigital as RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideDigital,
        RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideShipping as RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideShipping,
        RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionShipping as RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionShipping,
        RequestedSessionModifyParamsLineItemDetail as RequestedSessionModifyParamsLineItemDetail,
        RequestedSessionModifyParamsPaymentMethodOptions as RequestedSessionModifyParamsPaymentMethodOptions,
        RequestedSessionModifyParamsPaymentMethodOptionsCard as RequestedSessionModifyParamsPaymentMethodOptionsCard,
    )
    from stripe.params.delegated_checkout._requested_session_order_list_params import (
        RequestedSessionOrderListParams as RequestedSessionOrderListParams,
    )
    from stripe.params.delegated_checkout._requested_session_retrieve_params import (
        RequestedSessionRetrieveParams as RequestedSessionRetrieveParams,
    )
    from stripe.params.delegated_checkout._requested_session_update_params import (
        RequestedSessionUpdateParams as RequestedSessionUpdateParams,
        RequestedSessionUpdateParamsDiscounts as RequestedSessionUpdateParamsDiscounts,
        RequestedSessionUpdateParamsFulfillmentDetails as RequestedSessionUpdateParamsFulfillmentDetails,
        RequestedSessionUpdateParamsFulfillmentDetailsAddress as RequestedSessionUpdateParamsFulfillmentDetailsAddress,
        RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOption as RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOption,
        RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionDigital as RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionDigital,
        RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverride as RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverride,
        RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideDigital as RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideDigital,
        RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideShipping as RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideShipping,
        RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionShipping as RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionShipping,
        RequestedSessionUpdateParamsLineItemDetail as RequestedSessionUpdateParamsLineItemDetail,
        RequestedSessionUpdateParamsPaymentMethodOptions as RequestedSessionUpdateParamsPaymentMethodOptions,
        RequestedSessionUpdateParamsPaymentMethodOptionsCard as RequestedSessionUpdateParamsPaymentMethodOptionsCard,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "OrderRetrieveParams": (
        "stripe.params.delegated_checkout._order_retrieve_params",
        False,
    ),
    "RequestedSessionConfirmParams": (
        "stripe.params.delegated_checkout._requested_session_confirm_params",
        False,
    ),
    "RequestedSessionConfirmParamsAffiliateAttribution": (
        "stripe.params.delegated_checkout._requested_session_confirm_params",
        False,
    ),
    "RequestedSessionConfirmParamsAffiliateAttributionSource": (
        "stripe.params.delegated_checkout._requested_session_confirm_params",
        False,
    ),
    "RequestedSessionConfirmParamsBuyerConsents": (
        "stripe.params.delegated_checkout._requested_session_confirm_params",
        False,
    ),
    "RequestedSessionConfirmParamsBuyerConsentsMarketing": (
        "stripe.params.delegated_checkout._requested_session_confirm_params",
        False,
    ),
    "RequestedSessionConfirmParamsBuyerConsentsMarketingConsent": (
        "stripe.params.delegated_checkout._requested_session_confirm_params",
        False,
    ),
    "RequestedSessionConfirmParamsRiskDetails": (
        "stripe.params.delegated_checkout._requested_session_confirm_params",
        False,
    ),
    "RequestedSessionConfirmParamsRiskDetailsClientDeviceMetadataDetails": (
        "stripe.params.delegated_checkout._requested_session_confirm_params",
        False,
    ),
    "RequestedSessionCreateParams": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsAffiliateAttribution": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsAffiliateAttributionSource": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsDiscounts": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsFulfillmentDetails": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsFulfillmentDetailsAddress": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsLineItemDetail": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsPaymentMethodOptions": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsPaymentMethodOptionsCard": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionCreateParamsSellerDetails": (
        "stripe.params.delegated_checkout._requested_session_create_params",
        False,
    ),
    "RequestedSessionExpireParams": (
        "stripe.params.delegated_checkout._requested_session_expire_params",
        False,
    ),
    "RequestedSessionListOrdersParams": (
        "stripe.params.delegated_checkout._requested_session_list_orders_params",
        False,
    ),
    "RequestedSessionModifyParams": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsDiscounts": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsFulfillmentDetails": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsFulfillmentDetailsAddress": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOption": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionDigital": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverride": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideDigital": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideShipping": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionShipping": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsLineItemDetail": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsPaymentMethodOptions": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionModifyParamsPaymentMethodOptionsCard": (
        "stripe.params.delegated_checkout._requested_session_modify_params",
        False,
    ),
    "RequestedSessionOrderListParams": (
        "stripe.params.delegated_checkout._requested_session_order_list_params",
        False,
    ),
    "RequestedSessionRetrieveParams": (
        "stripe.params.delegated_checkout._requested_session_retrieve_params",
        False,
    ),
    "RequestedSessionUpdateParams": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsDiscounts": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsFulfillmentDetails": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsFulfillmentDetailsAddress": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOption": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionDigital": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverride": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideDigital": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideShipping": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsFulfillmentDetailsSelectedFulfillmentOptionShipping": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsLineItemDetail": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsPaymentMethodOptions": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
    "RequestedSessionUpdateParamsPaymentMethodOptionsCard": (
        "stripe.params.delegated_checkout._requested_session_update_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._stripe_object import StripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, List, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.radar._customer_evaluation_create_params import (
        CustomerEvaluationCreateParams,
    )
    from stripe.params.radar._customer_evaluation_modify_params import (
        CustomerEvaluationModifyParams,
    )
    from stripe.params.radar._customer_evaluation_retrieve_params import (
        CustomerEvaluationRetrieveParams,
    )


class CustomerEvaluation(
    CreateableAPIResource["CustomerEvaluation"],
    UpdateableAPIResource["CustomerEvaluation"],
):
    """
    Customer Evaluation resource returned by the Radar Customer Evaluations API.
    """

    OBJECT_NAME: ClassVar[Literal["radar.customer_evaluation"]] = (
        "radar.customer_evaluation"
    )

    class Event(StripeObject):
        class LoginFailed(StripeObject):
            reason: str
            """
            The reason why this login failed.
            """

        class RegistrationFailed(StripeObject):
            reason: str
            """
            The reason why this registration failed.
            """

        login_failed: Optional[LoginFailed]
        """
        Data about a failed login event.
        """
        occurred_at: int
        """
        Time at which the event occurred. Measured in seconds since the Unix epoch.
        """
        registration_failed: Optional[RegistrationFailed]
        """
        Data about a failed registration event.
        """
        type: str
        """
        The type of event that occurred.
        """
        _inner_class_types = {
            "login_failed": LoginFailed,
            "registration_failed": RegistrationFailed,
        }

    class Signals(StripeObject):
        class AccountSharing(StripeObject):
            evaluated_at: int
            """
            Time at which the signal was evaluated. Measured in seconds since the Unix epoch.
            """
            risk_level: Optional[
                Literal[
                    "elevated",
                    "highest",
                    "low",
                    "normal",
                    "not_assessed",
                    "unknown",
                ]
            ]
            """
            The risk level for this signal.
            """
            score: float
            """
            Score for this signal (between 0.0 and 100.0).
            """

        class MultiAccounting(StripeObject):
            evaluated_at: int
            """
            Time at which the signal was evaluated. Measured in seconds since the Unix epoch.
            """
            risk_level: Optional[
                Literal[
                    "elevated",
                    "highest",
                    "low",
                    "normal",
                    "not_assessed",
                    "unknown",
                ]
            ]
            """
            The risk level for this signal.
            """
            score: float
            """
            Score for this signal (between 0.0 and 100.0).
            """

        account_sharing: Optional[AccountSharing]
        multi_accounting: Optional[MultiAccounting]
        _inner_class_types = {
            "account_sharing": AccountSharing,
            "multi_accounting": MultiAccounting,
        }

    created_at: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    customer: Optional[str]
    """
    The ID of the Customer to associate with this CustomerEvaluation.
    """
    event_type: Literal["login", "registration"]
    """
    The type of evaluation event.
    """
    events: Optional[List[Event]]
    """
    A list of events that have been reported on this customer evaluation.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["radar.customer_evaluation"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    signals: Optional[Signals]
    """
    A hash of signal objects providing Radar's evaluation of the customer.
    """
    status: Optional[str]
    """
    The outcome status reported for this evaluation: allowed, restricted, or blocked.
    """

    @classmethod
    def create(
        cls, **params: Unpack["CustomerEvaluationCreateParams"]
    ) -> "CustomerEvaluation":
        """
        Creates a new CustomerEvaluation object.
        """
        return cast(
            "CustomerEvaluation",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["CustomerEvaluationCreateParams"]
    ) -> "CustomerEvaluation":
        """
        Creates a new CustomerEvaluation object.
        """
        return cast(
            "CustomerEvaluation",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["CustomerEvaluationModifyParams"]
    ) -> "CustomerEvaluation":
        """
        Reports an event on a CustomerEvaluation object.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "CustomerEvaluation",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["CustomerEvaluationModifyParams"]
    ) -> "CustomerEvaluation":
        """
        Reports an event on a CustomerEvaluation object.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "CustomerEvaluation",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["CustomerEvaluationRetrieveParams"]
    ) -> "CustomerEvaluation":
        """
        Retrieves an CustomerEvaluation object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["CustomerEvaluationRetrieveParams"]
    ) -> "CustomerEvaluation":
        """
        Retrieves an CustomerEvaluation object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"events": Event, "signals": Signals}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class StatementListParams(TypedDict):
    limit: NotRequired[int]
    """
    The maximum number of results to return.
    """
    order_by: NotRequired[Literal["created", "period_end_date"]]
    """
    The field by which to sort results. Defaults to "created".
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from typing import ClassVar, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._account_link_create_params import (
        AccountLinkCreateParams,
    )


class AccountLink(CreateableAPIResource["AccountLink"]):
    """
    Account Links are the means by which a Connect platform grants a connected account permission to access
    Stripe-hosted applications, such as Connect Onboarding.

    Related guide: [Connect Onboarding](https://docs.stripe.com/connect/custom/hosted-onboarding)
    """

    OBJECT_NAME: ClassVar[Literal["account_link"]] = "account_link"
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    expires_at: int
    """
    The timestamp at which this account link will expire.
    """
    object: Literal["account_link"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    url: str
    """
    The URL for the account link.
    """

    @classmethod
    def create(
        cls, **params: Unpack["AccountLinkCreateParams"]
    ) -> "AccountLink":
        """
        Creates an AccountLink object that includes a single-use Stripe URL that the platform can redirect their user to in order to take them through the Connect Onboarding flow.
        """
        return cast(
            "AccountLink",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["AccountLinkCreateParams"]
    ) -> "AccountLink":
        """
        Creates an AccountLink object that includes a single-use Stripe URL that the platform can redirect their user to in order to take them through the Connect Onboarding flow.
        """
        return cast(
            "AccountLink",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._profile import Profile
    from stripe.delegated_checkout._order import Order
    from stripe.params.delegated_checkout._requested_session_confirm_params import (
        RequestedSessionConfirmParams,
    )
    from stripe.params.delegated_checkout._requested_session_create_params import (
        RequestedSessionCreateParams,
    )
    from stripe.params.delegated_checkout._requested_session_expire_params import (
        RequestedSessionExpireParams,
    )
    from stripe.params.delegated_checkout._requested_session_list_orders_params import (
        RequestedSessionListOrdersParams,
    )
    from stripe.params.delegated_checkout._requested_session_modify_params import (
        RequestedSessionModifyParams,
    )
    from stripe.params.delegated_checkout._requested_session_retrieve_params import (
        RequestedSessionRetrieveParams,
    )
    from stripe.shared_payment._issued_token import IssuedToken


class RequestedSession(
    CreateableAPIResource["RequestedSession"],
    UpdateableAPIResource["RequestedSession"],
):
    """
    A requested session is a session that has been requested by a customer.
    """

    OBJECT_NAME: ClassVar[Literal["delegated_checkout.requested_session"]] = (
        "delegated_checkout.requested_session"
    )

    class AffiliateAttribution(StripeObject):
        class Source(StripeObject):
            platform: Optional[str]
            """
            The platform of the attribution source.
            """
            type: Literal["platform", "url"]
            """
            The type of the attribution source.
            """
            url: Optional[str]
            """
            The URL of the attribution source.
            """

        campaign_id: Optional[str]
        """
        Agent-scoped campaign identifier.
        """
        creative_id: Optional[str]
        """
        Agent-scoped creative identifier.
        """
        expires_at: int
        """
        Timestamp when the attribution token expires.
        """
        identification_token: str
        """
        Agent-issued secret to validate the legitimacy of the source of this data.
        """
        issued_at: int
        """
        Timestamp for when the attribution token was issued.
        """
        provider: str
        """
        Identifier for the attribution agent / affiliate network namespace.
        """
        publisher_id: Optional[str]
        """
        Agent-scoped affiliate/publisher identifier.
        """
        shared_metadata: Optional[UntypedStripeObject[str]]
        """
        Freeform key/value pairs for additional non-sensitive per-agent data.
        """
        source: Optional[Source]
        """
        Context about where the attribution originated.
        """
        sub_id: Optional[str]
        """
        Agent-scoped sub-tracking identifier.
        """
        touchpoint: Literal["first", "last"]
        """
        Whether this is the first or last touchpoint.
        """
        _inner_class_types = {"source": Source}

    class BuyerConsents(StripeObject):
        class Marketing(StripeObject):
            class Consent(StripeObject):
                channel: Literal["email", "sms"]
                """
                The marketing consent channel.
                """
                status: Literal["granted", "none"]
                """
                The consent status. 'granted' means the buyer opted in, 'none' means they did not.
                """

            class Option(StripeObject):
                channel: Literal["email", "sms"]
                """
                The marketing channel type.
                """
                description: str
                """
                The description of the marketing consent option.
                """
                privacy_policy_url: str
                """
                The privacy policy URL for this marketing channel.
                """

            consents: Optional[List[Consent]]
            """
            The buyer's marketing consent choices.
            """
            options: Optional[List[Option]]
            """
            The available marketing consent options.
            """
            _inner_class_types = {"consents": Consent, "options": Option}

        marketing: Optional[Marketing]
        """
        The marketing consent options.
        """
        _inner_class_types = {"marketing": Marketing}

    class Discounts(StripeObject):
        class Applied(StripeObject):
            amount_off: Optional[int]
            """
            The amount off provided by this discount.
            """
            code: str
            """
            The discount code.
            """
            currency: Optional[str]
            """
            The currency of the discount amount.
            """
            key: str
            """
            The unique key of the applied discount.
            """
            name: str
            """
            The display name of the discount.
            """
            percent_off: Optional[float]
            """
            The percentage off provided by this discount.
            """
            type: Literal["cart", "fulfillment"]
            """
            The type of discount.
            """

        class Invalid(StripeObject):
            code: str
            """
            The discount code that was invalid.
            """
            reason: str
            """
            The reason the discount code is invalid.
            """

        applied: Optional[List[Applied]]
        """
        The list of successfully applied discounts.
        """
        invalid: Optional[List[Invalid]]
        """
        The list of discount codes that could not be applied.
        """
        _inner_class_types = {"applied": Applied, "invalid": Invalid}

    class FulfillmentDetails(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: Optional[str]
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1, such as the street, PO Box, or company name.
            """
            line2: Optional[str]
            """
            Address line 2, such as the apartment, suite, unit, or building.
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """

        class FulfillmentOption(StripeObject):
            class Digital(StripeObject):
                class DigitalOption(StripeObject):
                    description: Optional[str]
                    """
                    The description of the digital fulfillment option.
                    """
                    digital_amount: int
                    """
                    The digital amount of the digital fulfillment option.
                    """
                    display_name: str
                    """
                    The display name of the digital fulfillment option.
                    """
                    key: str
                    """
                    The key of the digital fulfillment option.
                    """
                    line_item_keys: Optional[List[str]]
                    """
                    The line item keys associated with this digital fulfillment option.
                    """

                digital_options: Optional[List[DigitalOption]]
                """
                The digital options.
                """
                _inner_class_types = {"digital_options": DigitalOption}

            class Shipping(StripeObject):
                class ShippingOption(StripeObject):
                    description: Optional[str]
                    """
                    The description of the shipping option.
                    """
                    display_name: str
                    """
                    The display name of the shipping option.
                    """
                    earliest_delivery_time: Optional[int]
                    """
                    The earliest delivery time of the shipping option.
                    """
                    key: str
                    """
                    The key of the shipping option.
                    """
                    latest_delivery_time: Optional[int]
                    """
                    The latest delivery time of the shipping option.
                    """
                    line_item_keys: Optional[List[str]]
                    """
                    The line item keys associated with this shipping option.
                    """
                    shipping_amount: int
                    """
                    The shipping amount of the shipping option.
                    """

                shipping_options: Optional[List[ShippingOption]]
                """
                The shipping options.
                """
                _inner_class_types = {"shipping_options": ShippingOption}

            digital: Optional[Digital]
            """
            The digital fulfillment option.
            """
            shipping: Optional[Shipping]
            """
            The shipping option.
            """
            type: Literal["digital", "shipping"]
            """
            The type of the fulfillment option.
            """
            _inner_class_types = {"digital": Digital, "shipping": Shipping}

        class SelectedFulfillmentOption(StripeObject):
            class Digital(StripeObject):
                digital_option: Optional[str]
                """
                The digital option.
                """

            class Shipping(StripeObject):
                shipping_option: Optional[str]
                """
                The shipping option.
                """

            digital: Optional[Digital]
            """
            The digital fulfillment option.
            """
            shipping: Optional[Shipping]
            """
            The shipping option.
            """
            type: Literal["digital", "shipping"]
            """
            The type of the selected fulfillment option.
            """
            _inner_class_types = {"digital": Digital, "shipping": Shipping}

        class SelectedFulfillmentOptionOverride(StripeObject):
            class Digital(StripeObject):
                digital_option: Optional[str]
                """
                The digital option.
                """

            class Shipping(StripeObject):
                shipping_option: Optional[str]
                """
                The shipping option.
                """

            digital: Optional[Digital]
            """
            The digital fulfillment option.
            """
            line_item_keys: Optional[List[str]]
            """
            The line items this fulfillment option applies to.
            """
            shipping: Optional[Shipping]
            """
            The shipping option.
            """
            type: Literal["digital", "shipping"]
            """
            The type of the selected fulfillment option.
            """
            _inner_class_types = {"digital": Digital, "shipping": Shipping}

        address: Optional[Address]
        """
        The fulfillment address.
        """
        email: Optional[str]
        """
        The email address for the fulfillment details.
        """
        fulfillment_options: Optional[List[FulfillmentOption]]
        """
        The fulfillment options.
        """
        name: Optional[str]
        """
        The name for the fulfillment details.
        """
        phone: Optional[str]
        """
        The phone number for the fulfillment details.
        """
        selected_fulfillment_option: Optional[SelectedFulfillmentOption]
        """
        The selected fulfillment option.
        """
        selected_fulfillment_option_overrides: Optional[
            List[SelectedFulfillmentOptionOverride]
        ]
        """
        Per-item fulfillment option overrides.
        """
        _inner_class_types = {
            "address": Address,
            "fulfillment_options": FulfillmentOption,
            "selected_fulfillment_option": SelectedFulfillmentOption,
            "selected_fulfillment_option_overrides": SelectedFulfillmentOptionOverride,
        }

    class LineItemDetail(StripeObject):
        class ProductDetails(StripeObject):
            class CustomAttribute(StripeObject):
                display_name: str
                """
                The display name of the custom attribute.
                """
                value: str
                """
                The value of the custom attribute.
                """

            class Disclosure(StripeObject):
                content: str
                """
                The content of the disclosure.
                """
                content_type: Literal["link", "markdown", "plain"]
                """
                The content type of the disclosure.
                """
                type: Literal["disclaimer"]
                """
                The type of disclosure.
                """

            custom_attributes: Optional[List[CustomAttribute]]
            """
            Custom attributes for the product.
            """
            description: Optional[str]
            """
            The description of the product.
            """
            disclosures: Optional[List[Disclosure]]
            """
            Disclosures for the product.
            """
            images: Optional[List[str]]
            """
            The images of the product.
            """
            title: str
            """
            The title of the product.
            """
            _inner_class_types = {
                "custom_attributes": CustomAttribute,
                "disclosures": Disclosure,
            }

        amount_discount: int
        """
        The total discount for this line item. If no discount were applied, defaults to 0.
        """
        amount_sale: Optional[int]
        """
        The sale amount for this line item.
        """
        amount_subtotal: int
        """
        The total before any discounts or taxes are applied.
        """
        fulfillment_type: str
        """
        The fulfillment type of the line item.
        """
        key: str
        """
        The key of the line item.
        """
        product_details: Optional[ProductDetails]
        quantity: int
        """
        The quantity of the line item.
        """
        sku_id: str
        """
        The SKU ID of the line item.
        """
        unit_amount: int
        """
        The per-unit amount of the item before any discounts or taxes are applied.
        """
        _inner_class_types = {"product_details": ProductDetails}

    class OrderDetails(StripeObject):
        order_id: Optional[str]
        """
        The seller's order identifier.
        """
        order_status_url: Optional[str]
        """
        The URL to the order status.
        """

    class PaymentMethodOptions(StripeObject):
        class Card(StripeObject):
            brands_blocked: Optional[
                List[Literal["american_express", "mastercard", "visa"]]
            ]
            """
            The card brands blocked by the agent.
            """

        card: Optional[Card]
        """
        Card-specific payment method options.
        """
        displayable_card_brands: Optional[
            List[Literal["american_express", "mastercard", "visa"]]
        ]
        """
        The computed displayable card brands.
        """
        displayable_payment_method_types: Optional[
            List[Literal["affirm", "card", "klarna"]]
        ]
        """
        The computed displayable payment method types.
        """
        excluded_payment_method_types: Optional[
            List[Literal["affirm", "card", "klarna"]]
        ]
        """
        The payment method types excluded by the agent.
        """
        _inner_class_types = {"card": Card}

    class PaymentMethodPreview(StripeObject):
        class BillingDetails(StripeObject):
            class Address(StripeObject):
                city: str
                """
                City, district, suburb, town, or village.
                """
                country: str
                """
                Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                """
                line1: str
                """
                Address line 1, such as the street, PO Box, or company name.
                """
                line2: Optional[str]
                """
                Address line 2, such as the apartment, suite, unit, or building.
                """
                postal_code: str
                """
                ZIP or postal code.
                """
                state: str
                """
                State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                """

            address: Optional[Address]
            """
            The billing address.
            """
            email: Optional[str]
            """
            The email address for the billing details.
            """
            name: Optional[str]
            """
            The name for the billing details.
            """
            phone: Optional[str]
            """
            The phone number for the billing details.
            """
            _inner_class_types = {"address": Address}

        class Card(StripeObject):
            exp_month: int
            """
            The expiry month of the card.
            """
            exp_year: int
            """
            The expiry year of the card.
            """
            last4: str
            """
            The last 4 digits of the card number.
            """

        billing_details: Optional[BillingDetails]
        """
        The billing details of the payment method.
        """
        card: Optional[Card]
        """
        The card details of the payment method.
        """
        type: str
        """
        The type of the payment method.
        """
        _inner_class_types = {"billing_details": BillingDetails, "card": Card}

    class RiskDetails(StripeObject):
        class ClientDeviceMetadataDetails(StripeObject):
            radar_session: Optional[str]
            """
            The radar session for the client device.
            """
            referrer: Optional[str]
            """
            The referrer of the client device.
            """
            remote_ip: Optional[str]
            """
            The remote IP address of the client device.
            """
            time_on_page_ms: Optional[int]
            """
            The time spent on the page by the client device.
            """
            user_agent: Optional[str]
            """
            The user agent of the client device.
            """

        client_device_metadata_details: Optional[ClientDeviceMetadataDetails]
        """
        The risk metadata for the client device.
        """
        _inner_class_types = {
            "client_device_metadata_details": ClientDeviceMetadataDetails,
        }

    class SellerDetails(StripeObject):
        class MarketplaceSellerDetails(StripeObject):
            pass

        card_brands: Optional[
            List[Literal["american_express", "mastercard", "visa"]]
        ]
        """
        The card brands supported by the seller.
        """
        marketplace_seller_details: Optional[MarketplaceSellerDetails]
        """
        The marketplace seller details.
        """
        network_profile: ExpandableField["Profile"]
        """
        The network profile of the seller.
        """
        payment_method_types: Optional[
            List[Literal["affirm", "card", "klarna"]]
        ]
        """
        The payment method types supported by the seller.
        """
        privacy_notice_url: Optional[str]
        """
        The URL to the seller's privacy notice.
        """
        return_policy_url: Optional[str]
        """
        The URL to the seller's return policy.
        """
        store_policy_url: Optional[str]
        """
        The URL to the seller's store policy.
        """
        terms_of_service_url: Optional[str]
        """
        The URL to the seller's terms of service.
        """
        _inner_class_types = {
            "marketplace_seller_details": MarketplaceSellerDetails,
        }

    class TotalDetails(StripeObject):
        class ApplicableFee(StripeObject):
            amount: int
            """
            The amount of the applicable fee.
            """
            description: Optional[str]
            """
            The description of the applicable fee.
            """
            display_name: str
            """
            The display name of the applicable fee.
            """

        class Breakdown(StripeObject):
            class Discount(StripeObject):
                amount: int
                """
                The amount this discount contributed to the total discount.
                """
                key: str
                """
                The key of the applied discount.
                """

            discounts: Optional[List[Discount]]
            """
            The breakdown of discounts applied to the session.
            """
            _inner_class_types = {"discounts": Discount}

        amount_cart_discount: Optional[int]
        """
        The amount of order-level discounts applied to the cart. The total discount amount for this session can be computed by summing the cart discount and the item discounts.
        """
        amount_discount: Optional[int]
        """
        The total discount amount from discount codes across the session.
        """
        amount_fulfillment: Optional[int]
        """
        The amount fulfillment of the total details.
        """
        amount_items_discount: Optional[int]
        """
        The amount of item-level discounts applied to the cart. The total discount amount for this session can be computed by summing the cart discount and the item discounts.
        """
        amount_sale: Optional[int]
        """
        The total sale amount across the session.
        """
        amount_tax: Optional[int]
        """
        The amount tax of the total details.
        """
        applicable_fees: Optional[List[ApplicableFee]]
        """
        The applicable fees of the total details.
        """
        breakdown: Optional[Breakdown]
        """
        The breakdown of discounts applied to the session.
        """
        _inner_class_types = {
            "applicable_fees": ApplicableFee,
            "breakdown": Breakdown,
        }

    affiliate_attributions: Optional[List[AffiliateAttribution]]
    """
    Affiliate attribution data associated with this requested session.
    """
    amount_subtotal: Optional[int]
    """
    The subtotal amount of the requested session.
    """
    amount_total: Optional[int]
    """
    The total amount of the requested session.
    """
    buyer_consents: Optional[BuyerConsents]
    """
    The buyer consent options for this requested session, including marketing preferences.
    """
    created_at: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    customer: Optional[str]
    """
    The customer for this requested session.
    """
    discounts: Optional[Discounts]
    """
    The discounts applied to and rejected from this requested session.
    """
    expires_at: int
    """
    Time at which the requested session expires. Measured in seconds since the Unix epoch.
    """
    fulfillment_details: Optional[FulfillmentDetails]
    """
    The details of the fulfillment.
    """
    id: str
    """
    Unique identifier for the object.
    """
    line_item_details: List[LineItemDetail]
    """
    The line items to be purchased.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["delegated_checkout.requested_session"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    order_details: Optional[OrderDetails]
    """
    The details of the order.
    """
    payment_method: Optional[str]
    """
    The payment method used for the requested session.
    """
    payment_method_options: Optional[PaymentMethodOptions]
    """
    The payment method options for this requested session.
    """
    payment_method_preview: Optional[PaymentMethodPreview]
    """
    The preview of the payment method to be created when the requested session is confirmed.
    """
    risk_details: Optional[RiskDetails]
    """
    The risk details of the requested session.
    """
    seller_details: SellerDetails
    setup_future_usage: Optional[Literal["on_session"]]
    """
    Whether or not the payment method should be saved for future use.
    """
    shared_metadata: Optional[UntypedStripeObject[str]]
    """
    The metadata shared with the seller.
    """
    shared_payment_issued_token: Optional[ExpandableField["IssuedToken"]]
    """
    The SPT used for payment.
    """
    status: Literal["completed", "expired", "open", "requires_action"]
    """
    The status of the requested session.
    """
    total_details: TotalDetails
    updated_at: int
    """
    Time at which the object was last updated. Measured in seconds since the Unix epoch.
    """

    @classmethod
    def _cls_confirm(
        cls,
        requested_session: str,
        **params: Unpack["RequestedSessionConfirmParams"],
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        return cast(
            "RequestedSession",
            cls._static_request(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/confirm".format(
                    requested_session=sanitize_id(requested_session)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def confirm(
        requested_session: str,
        **params: Unpack["RequestedSessionConfirmParams"],
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        ...

    @overload
    def confirm(
        self, **params: Unpack["RequestedSessionConfirmParams"]
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        ...

    @class_method_variant("_cls_confirm")
    def confirm(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["RequestedSessionConfirmParams"]
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        return cast(
            "RequestedSession",
            self._request(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/confirm".format(
                    requested_session=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_confirm_async(
        cls,
        requested_session: str,
        **params: Unpack["RequestedSessionConfirmParams"],
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        return cast(
            "RequestedSession",
            await cls._static_request_async(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/confirm".format(
                    requested_session=sanitize_id(requested_session)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def confirm_async(
        requested_session: str,
        **params: Unpack["RequestedSessionConfirmParams"],
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        ...

    @overload
    async def confirm_async(
        self, **params: Unpack["RequestedSessionConfirmParams"]
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        ...

    @class_method_variant("_cls_confirm_async")
    async def confirm_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["RequestedSessionConfirmParams"]
    ) -> "RequestedSession":
        """
        Confirms a requested session
        """
        return cast(
            "RequestedSession",
            await self._request_async(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/confirm".format(
                    requested_session=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def create(
        cls, **params: Unpack["RequestedSessionCreateParams"]
    ) -> "RequestedSession":
        """
        Creates a requested session
        """
        return cast(
            "RequestedSession",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["RequestedSessionCreateParams"]
    ) -> "RequestedSession":
        """
        Creates a requested session
        """
        return cast(
            "RequestedSession",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_expire(
        cls,
        requested_session: str,
        **params: Unpack["RequestedSessionExpireParams"],
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        return cast(
            "RequestedSession",
            cls._static_request(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/expire".format(
                    requested_session=sanitize_id(requested_session)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def expire(
        requested_session: str,
        **params: Unpack["RequestedSessionExpireParams"],
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        ...

    @overload
    def expire(
        self, **params: Unpack["RequestedSessionExpireParams"]
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        ...

    @class_method_variant("_cls_expire")
    def expire(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["RequestedSessionExpireParams"]
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        return cast(
            "RequestedSession",
            self._request(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/expire".format(
                    requested_session=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_expire_async(
        cls,
        requested_session: str,
        **params: Unpack["RequestedSessionExpireParams"],
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        return cast(
            "RequestedSession",
            await cls._static_request_async(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/expire".format(
                    requested_session=sanitize_id(requested_session)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def expire_async(
        requested_session: str,
        **params: Unpack["RequestedSessionExpireParams"],
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        ...

    @overload
    async def expire_async(
        self, **params: Unpack["RequestedSessionExpireParams"]
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        ...

    @class_method_variant("_cls_expire_async")
    async def expire_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["RequestedSessionExpireParams"]
    ) -> "RequestedSession":
        """
        Expires a requested session
        """
        return cast(
            "RequestedSession",
            await self._request_async(
                "post",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/expire".format(
                    requested_session=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def _cls_list_orders(
        cls,
        requested_session: str,
        **params: Unpack["RequestedSessionListOrdersParams"],
    ) -> ListObject["Order"]:
        """
        Lists orders for a delegated checkout requested session.
        """
        return cast(
            ListObject["Order"],
            cls._static_request(
                "get",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/orders".format(
                    requested_session=sanitize_id(requested_session)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def list_orders(
        requested_session: str,
        **params: Unpack["RequestedSessionListOrdersParams"],
    ) -> ListObject["Order"]:
        """
        Lists orders for a delegated checkout requested session.
        """
        ...

    @overload
    def list_orders(
        self, **params: Unpack["RequestedSessionListOrdersParams"]
    ) -> ListObject["Order"]:
        """
        Lists orders for a delegated checkout requested session.
        """
        ...

    @class_method_variant("_cls_list_orders")
    def list_orders(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["RequestedSessionListOrdersParams"]
    ) -> ListObject["Order"]:
        """
        Lists orders for a delegated checkout requested session.
        """
        return cast(
            ListObject["Order"],
            self._request(
                "get",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/orders".format(
                    requested_session=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_list_orders_async(
        cls,
        requested_session: str,
        **params: Unpack["RequestedSessionListOrdersParams"],
    ) -> ListObject["Order"]:
        """
        Lists orders for a delegated checkout requested session.
        """
        return cast(
            ListObject["Order"],
            await cls._static_request_async(
                "get",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/orders".format(
                    requested_session=sanitize_id(requested_session)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def list_orders_async(
        requested_session: str,
        **params: Unpack["RequestedSessionListOrdersParams"],
    ) -> ListObject["Order"]:
        """
        Lists orders for a delegated checkout requested session.
        """
        ...

    @overload
    async def list_orders_async(
        self, **params: Unpack["RequestedSessionListOrdersParams"]
    ) -> ListObject["Order"]:
        """
        Lists orders for a delegated checkout requested session.
        """
        ...

    @class_method_variant("_cls_list_orders_async")
    async def list_orders_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["RequestedSessionListOrdersParams"]
    ) -> ListObject["Order"]:
        """
        Lists orders for a delegated checkout requested session.
        """
        return cast(
            ListObject["Order"],
            await self._request_async(
                "get",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/orders".format(
                    requested_session=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["RequestedSessionModifyParams"]
    ) -> "RequestedSession":
        """
        Updates a requested session
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "RequestedSession",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["RequestedSessionModifyParams"]
    ) -> "RequestedSession":
        """
        Updates a requested session
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "RequestedSession",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["RequestedSessionRetrieveParams"]
    ) -> "RequestedSession":
        """
        Retrieves a requested session
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["RequestedSessionRetrieveParams"]
    ) -> "RequestedSession":
        """
        Retrieves a requested session
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "affiliate_attributions": AffiliateAttribution,
        "buyer_consents": BuyerConsents,
        "discounts": Discounts,
        "fulfillment_details": FulfillmentDetails,
        "line_item_details": LineItemDetail,
        "order_details": OrderDetails,
        "payment_method_options": PaymentMethodOptions,
        "payment_method_preview": PaymentMethodPreview,
        "risk_details": RiskDetails,
        "seller_details": SellerDetails,
        "total_details": TotalDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict
from typing_extensions import NotRequired, TypedDict


class RateCardSubscriptionCreateParams(TypedDict):
    billing_cadence: str
    """
    The ID of the Billing Cadence.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    rate_card: str
    """
    The ID of the Rate Card.
    """
    rate_card_version: NotRequired[str]
    """
    The ID of the Rate Card Version. If not specified, defaults to the "live_version" of the Rate Card at the time of creation.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class CollectionSettingVersion(StripeObject):
    """
    A Collection Setting Version is a specific configuration of a CollectionSetting at a point in time. Collection Setting Versions enable you to track changes to payment collection settings over time and manage which version is active for new billing operations.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.collection_setting_version"]] = (
        "v2.billing.collection_setting_version"
    )

    class EmailDelivery(StripeObject):
        class PaymentDue(StripeObject):
            enabled: bool
            """
            If true an email for the invoice would be generated and sent out.
            """
            include_payment_link: bool
            """
            If true the payment link to hosted invoice page would be included in email and PDF of the invoice.
            """

        payment_due: Optional[PaymentDue]
        """
        Controls emails for when the payment is due. For example after the invoice is finalized and transitions to Open state.
        """
        _inner_class_types = {"payment_due": PaymentDue}

    class PaymentMethodOptions(StripeObject):
        class AcssDebit(StripeObject):
            class MandateOptions(StripeObject):
                transaction_type: Optional[Literal["business", "personal"]]
                """
                Transaction type of the mandate.
                """

            mandate_options: Optional[MandateOptions]
            """
            Additional fields for Mandate creation.
            """
            verification_method: Optional[
                Literal["automatic", "instant", "microdeposits"]
            ]
            """
            Verification method.
            """
            _inner_class_types = {"mandate_options": MandateOptions}

        class Bancontact(StripeObject):
            preferred_language: Optional[Literal["de", "en", "fr", "nl"]]
            """
            Preferred language of the Bancontact authorization page that the customer is redirected to.
            """

        class Card(StripeObject):
            class MandateOptions(StripeObject):
                amount: Optional[int]
                """
                Amount to be charged for future payments.
                """
                amount_type: Optional[Literal["fixed", "maximum"]]
                """
                The AmountType for the mandate. One of `fixed` or `maximum`.
                """
                description: Optional[str]
                """
                A description of the mandate that is meant to be displayed to the customer.
                """
                _field_encodings = {"amount": "int64_string"}

            mandate_options: Optional[MandateOptions]
            """
            Configuration options for setting up an eMandate for cards issued in India.
            """
            network: Optional[str]
            """
            Selected network to process the payment on. Depends on the available networks of the card.
            """
            request_three_d_secure: Optional[
                Literal["any", "automatic", "challenge"]
            ]
            """
            An advanced option 3D Secure. We strongly recommend that you rely on our SCA Engine to automatically prompt your customers
            for authentication based on risk level and [other requirements](https://docs.stripe.com/strong-customer-authentication).
            However, if you wish to request 3D Secure based on logic from your own fraud engine, provide this option.
            Read our guide on [manually requesting 3D Secure](https://docs.stripe.com/payments/3d-secure/authentication-flow#manual-three-ds) for more information on how this configuration interacts with Radar and our SCA Engine.
            """
            _inner_class_types = {"mandate_options": MandateOptions}

        class CustomerBalance(StripeObject):
            class BankTransfer(StripeObject):
                class EuBankTransfer(StripeObject):
                    country: Literal["BE", "DE", "ES", "FR", "IE", "NL"]
                    """
                    The desired country code of the bank account information.
                    """

                eu_bank_transfer: Optional[EuBankTransfer]
                """
                Configuration for `eu_bank_transfer` funding type. Required if `type` is `eu_bank_transfer`.
                """
                type: Optional[
                    Literal[
                        "eu_bank_transfer",
                        "gb_bank_transfer",
                        "jp_bank_transfer",
                        "mx_bank_transfer",
                        "us_bank_transfer",
                    ]
                ]
                """
                The bank transfer type that can be used for funding.
                """
                _inner_class_types = {"eu_bank_transfer": EuBankTransfer}

            bank_transfer: Optional[BankTransfer]
            """
            Configuration for the bank transfer funding type, if the `funding_type` is set to `bank_transfer`.
            """
            funding_type: Optional[Literal["bank_transfer"]]
            """
            The funding method type to be used when there are not enough funds in the customer balance. Currently the only supported value is `bank_transfer`.
            """
            _inner_class_types = {"bank_transfer": BankTransfer}

        class Konbini(StripeObject):
            pass

        class SepaDebit(StripeObject):
            pass

        class UsBankAccount(StripeObject):
            class FinancialConnections(StripeObject):
                class Filters(StripeObject):
                    account_subcategories: List[Literal["checking", "savings"]]
                    """
                    The account subcategories to use to filter for selectable accounts.
                    """

                filters: Optional[Filters]
                """
                Provide filters for the linked accounts that the customer can select for the payment method.
                """
                permissions: List[
                    Literal[
                        "balances",
                        "ownership",
                        "payment_method",
                        "transactions",
                    ]
                ]
                """
                The list of permissions to request. If this parameter is passed, the `payment_method` permission must be included.
                """
                prefetch: List[
                    Literal["balances", "ownership", "transactions"]
                ]
                """
                List of data features that you would like to retrieve upon account creation.
                """
                _inner_class_types = {"filters": Filters}

            financial_connections: FinancialConnections
            """
            Additional fields for Financial Connections Session creation.
            """
            verification_method: Literal[
                "automatic", "instant", "microdeposits"
            ]
            """
            Verification method.
            """
            _inner_class_types = {
                "financial_connections": FinancialConnections
            }

        acss_debit: Optional[AcssDebit]
        """
        This sub-hash contains details about the Canadian pre-authorized debit payment method options.
        """
        bancontact: Optional[Bancontact]
        """
        This sub-hash contains details about the Bancontact payment method.
        """
        card: Optional[Card]
        """
        This sub-hash contains details about the Card payment method options.
        """
        customer_balance: Optional[CustomerBalance]
        """
        This sub-hash contains details about the Bank transfer payment method options.
        """
        konbini: Optional[Konbini]
        """
        This sub-hash contains details about the Konbini payment method options.
        """
        sepa_debit: Optional[SepaDebit]
        """
        This sub-hash contains details about the SEPA Direct Debit payment method options.
        """
        us_bank_account: Optional[UsBankAccount]
        """
        This sub-hash contains details about the ACH direct debit payment method options.
        """
        _inner_class_types = {
            "acss_debit": AcssDebit,
            "bancontact": Bancontact,
            "card": Card,
            "customer_balance": CustomerBalance,
            "konbini": Konbini,
            "sepa_debit": SepaDebit,
            "us_bank_account": UsBankAccount,
        }

    collection_method: Optional[Literal["automatic", "send_invoice"]]
    """
    Either automatic, or send_invoice. When charging automatically, Stripe attempts to pay this
    bill at the end of the period using the payment method attached to the billing profile. When sending an invoice,
    Stripe emails your billing profile an invoice with payment instructions.
    Defaults to automatic.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    email_delivery: Optional[EmailDelivery]
    """
    Email delivery settings.
    """
    id: str
    """
    The ID of the CollectionSettingVersion object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.billing.collection_setting_version"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    payment_method_configuration: Optional[str]
    """
    The ID of the PaymentMethodConfiguration object, which controls which payment methods are displayed to your customers.
    """
    payment_method_options: Optional[PaymentMethodOptions]
    """
    Payment Method specific configuration stored on the object.
    """
    _inner_class_types = {
        "email_delivery": EmailDelivery,
        "payment_method_options": PaymentMethodOptions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, Optional
from typing_extensions import Literal


class Transaction(StripeObject):
    """
    Use Transactions to view changes to your FinancialAccount balance over time. Every flow that moves money, such as OutboundPayments or ReceivedCredits, will have one or more Transactions that describes how the flow impacted your balance. Note that while the FinancialAccount balance will always be up to date, be aware that Transactions and TransactionEntries are created shortly after to reflect changes.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.transaction"]] = (
        "v2.money_management.transaction"
    )

    class BalanceImpact(StripeObject):
        available: Amount
        """
        Impact to the available balance.
        """
        inbound_pending: Amount
        """
        Impact to the inbound_pending balance.
        """
        outbound_pending: Amount
        """
        Impact to the outbound_pending balance.
        """

    class Counterparty(StripeObject):
        name: Optional[str]
        """
        Name of the counterparty.
        """

    class Flow(StripeObject):
        account: Optional[str]
        """
        If applicable, the connected account associated with this Transaction's flow.
        """
        adjustment: Optional[str]
        """
        If applicable, the ID of the Adjustment that created this Transaction.
        """
        application_fee: Optional[str]
        """
        If applicable, the ID of the Application Fee that created this Transaction.
        """
        application_fee_refund: Optional[str]
        """
        If applicable, the ID of the Application Fee Refund that created this Transaction.
        """
        charge: Optional[str]
        """
        If applicable, the ID of the Charge that created this Transaction.
        """
        currency_conversion: Optional[str]
        """
        In the future, this will be the ID of the currency conversion that created this Transaction. For now, this field is always null.
        """
        debit_dispute: Optional[str]
        """
        If applicable, the ID of the DebitDispute that created this Transaction.
        """
        dispute: Optional[str]
        """
        If applicable, the ID of the Dispute that created this Transaction.
        """
        fee_transaction: Optional[str]
        """
        If applicable, the ID of the FeeTransaction that created this Transaction.
        """
        inbound_transfer: Optional[str]
        """
        If applicable, the ID of the InboundTransfer that created this Transaction.
        """
        issuing_authorization: Optional[str]
        """
        If applicable, the ID of the Issuing authorization that created this Transaction.
        """
        issuing_dispute: Optional[str]
        """
        If applicable, the ID of the Issuing dispute that created this Transaction.
        """
        issuing_transaction: Optional[str]
        """
        If applicable, the ID of the Issuing transaction that created this Transaction.
        """
        outbound_payment: Optional[str]
        """
        If applicable, the ID of the OutboundPayment that created this Transaction.
        """
        outbound_transfer: Optional[str]
        """
        If applicable, the ID of the OutboundTransfer that created this Transaction.
        """
        payout: Optional[str]
        """
        If applicable, the ID of the Payout that created this Transaction.
        """
        received_credit: Optional[str]
        """
        If applicable, the ID of the ReceivedCredit that created this Transaction.
        """
        received_debit: Optional[str]
        """
        If applicable, the ID of the ReceivedDebit that created this Transaction.
        """
        refund: Optional[str]
        """
        If applicable, the ID of the Refund that created this Transaction.
        """
        reserve_hold: Optional[str]
        """
        If applicable, the ID of the Reserve Hold that created this Transaction.
        """
        reserve_release: Optional[str]
        """
        If applicable, the ID of the Reserve Release that created this Transaction.
        """
        tax_fund: Optional[str]
        """
        If applicable, the ID of the TaxFund that created this Transaction.
        """
        topup: Optional[str]
        """
        If applicable, the ID of the Topup that created this Transaction.
        """
        transfer: Optional[str]
        """
        If applicable, the ID of the Transfer that created this Transaction.
        """
        transfer_reversal: Optional[str]
        """
        If applicable, the ID of the Transfer Reversal that created this Transaction.
        """
        treasury_credit_reversal: Optional[str]
        """
        If applicable, the ID of the Treasury CreditReversal that created this Transaction.
        """
        treasury_debit_reversal: Optional[str]
        """
        If applicable, the ID of the Treasury DebitReversal that created this Transaction.
        """
        treasury_inbound_transfer: Optional[str]
        """
        If applicable, the ID of the Treasury InboundTransfer that created this Transaction.
        """
        treasury_issuing_authorization: Optional[str]
        """
        If applicable, the ID of the Treasury IssuingAuthorization that created this Transaction.
        """
        treasury_outbound_payment: Optional[str]
        """
        If applicable, the ID of the Treasury OutboundPayment that created this Transaction.
        """
        treasury_outbound_transfer: Optional[str]
        """
        If applicable, the ID of the Treasury OutboundTransfer that created this Transaction.
        """
        treasury_received_credit: Optional[str]
        """
        If applicable, the ID of the Treasury ReceivedCredit that created this Transaction.
        """
        treasury_received_debit: Optional[str]
        """
        If applicable, the ID of the Treasury ReceivedDebit that created this Transaction.
        """
        type: Literal[
            "adjustment",
            "application_fee",
            "application_fee_refund",
            "charge",
            "currency_conversion",
            "debit_dispute",
            "dispute",
            "fee_transaction",
            "inbound_transfer",
            "issuing_authorization",
            "issuing_dispute",
            "issuing_transaction",
            "outbound_payment",
            "outbound_transfer",
            "payout",
            "received_credit",
            "received_debit",
            "refund",
            "reserve_hold",
            "reserve_release",
            "tax_fund",
            "topup",
            "transfer",
            "transfer_reversal",
            "treasury_credit_reversal",
            "treasury_debit_reversal",
            "treasury_inbound_transfer",
            "treasury_issuing_authorization",
            "treasury_other",
            "treasury_outbound_payment",
            "treasury_outbound_transfer",
            "treasury_received_credit",
            "treasury_received_debit",
        ]
        """
        Open Enum. Type of the flow that created the Transaction. The field matching this value will contain the ID of the flow.
        """

    class StatusTransitions(StripeObject):
        posted_at: Optional[str]
        """
        The time at which the Transaction became posted. Only present if status == posted.
        """
        void_at: Optional[str]
        """
        The time at which the Transaction became void. Only present if status == void.
        """

    amount: Amount
    """
    The amount of the Transaction.
    """
    balance_impact: BalanceImpact
    """
    The delta to the FinancialAccount's balance. The balance_impact for the Transaction is equal to sum of its
    TransactionEntries that have `effective_at`s in the past.
    """
    category: Literal[
        "adjustment",
        "advance",
        "anticipation_repayment",
        "balance_transfer",
        "climate_order_purchase",
        "climate_order_refund",
        "connect_collection_transfer",
        "connect_reserved_funds",
        "contribution",
        "currency_conversion",
        "debit_dispute",
        "dispute",
        "dispute_reversal",
        "financing_paydown",
        "financing_paydown_reversal",
        "inbound_payment",
        "inbound_payment_failure",
        "inbound_transfer",
        "inbound_transfer_reversal",
        "india_mdr_processing_fee",
        "issuing_authorization",
        "issuing_dispute",
        "issuing_dispute_fraud_liability_debit",
        "issuing_dispute_provisional_credit",
        "issuing_dispute_provisional_credit_reversal",
        "issuing_transaction",
        "minimum_balance_hold",
        "network_cost",
        "obligation",
        "outbound_payment",
        "outbound_payment_reversal",
        "outbound_transfer",
        "outbound_transfer_reversal",
        "partial_capture_reversal",
        "payment_method_passthrough_fee",
        "payment_network_reserved_funds",
        "platform_earning",
        "platform_earning_refund",
        "platform_fee",
        "platform_funded_credit_transaction",
        "received_credit",
        "received_credit_reversal",
        "received_debit",
        "received_debit_reversal",
        "refund",
        "refund_failure",
        "risk_reserved_funds",
        "stripe_balance_payment_debit",
        "stripe_balance_payment_debit_reversal",
        "stripe_fee",
        "stripe_fee_tax",
        "tax_fund",
        "tax_withholding",
        "transfer_reversal",
        "unreconciled_customer_funds",
    ]
    """
    Open Enum. A descriptive category used to classify the Transaction.
    """
    counterparty: Optional[Counterparty]
    """
    Counterparty to this Transaction.
    """
    created: str
    """
    Time at which the object was created. Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    description: Optional[str]
    """
    Description of this Transaction. When applicable, the description is copied from the Flow object at the time
    of transaction creation.
    """
    financial_account: str
    """
    Indicates the FinancialAccount affected by this Transaction.
    """
    flow: Optional[Flow]
    """
    Details about the Flow object that created the Transaction.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.money_management.transaction"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: Literal["pending", "posted", "void"]
    """
    Closed Enum. Current status of the Transaction.
    A Transaction is `pending` if either `balance_impact.inbound_pending` or `balance_impact.outbound_pending` is non-zero.
    A Transaction is `posted` if only `balance_impact.available` is non-zero.
    A Transaction is `void` if there is no balance impact.
    `posted` and `void` are terminal states, and no additional entries will be added to the Transaction.
    """
    status_transitions: StatusTransitions
    """
    Timestamps for when the Transaction transitioned to a particular status.
    """
    treasury_transaction: Optional[str]
    """
    The v1 Treasury transaction associated with this transaction.
    """
    _inner_class_types = {
        "balance_impact": BalanceImpact,
        "counterparty": Counterparty,
        "flow": Flow,
        "status_transitions": StatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._account import Account as AccountResource
    from stripe._customer import Customer
    from stripe.financial_connections._account import (
        Account as FinancialConnectionsAccountResource,
    )
    from stripe.params.financial_connections._session_create_params import (
        SessionCreateParams,
    )
    from stripe.params.financial_connections._session_retrieve_params import (
        SessionRetrieveParams,
    )


class Session(CreateableAPIResource["Session"]):
    """
    A Financial Connections Session is the secure way to programmatically launch the client-side Stripe.js modal that lets your users link their accounts.
    """

    OBJECT_NAME: ClassVar[Literal["financial_connections.session"]] = (
        "financial_connections.session"
    )

    class AccountHolder(StripeObject):
        account: Optional[ExpandableField["AccountResource"]]
        """
        The ID of the Stripe account that this account belongs to. Only available when `account_holder.type` is `account`.
        """
        customer: Optional[ExpandableField["Customer"]]
        """
        The ID for an Account representing a customer that this account belongs to. Only available when `account_holder.type` is `customer`.
        """
        customer_account: Optional[str]
        type: Literal["account", "customer"]
        """
        Type of account holder that this account belongs to.
        """

    class Filters(StripeObject):
        account_subcategories: Optional[
            List[
                Literal[
                    "checking",
                    "credit_card",
                    "line_of_credit",
                    "mortgage",
                    "savings",
                ]
            ]
        ]
        """
        Restricts the Session to subcategories of accounts that can be linked. Valid subcategories are: `checking`, `savings`, `mortgage`, `line_of_credit`, `credit_card`.
        """
        countries: Optional[List[str]]
        """
        List of countries from which to filter accounts.
        """
        institution: Optional[str]
        """
        Stripe ID of the institution with which the customer should be directed to log in.
        """

    class Hosted(StripeObject):
        delivery_method: Optional[Literal["email", "url"]]
        """
        How the user enters the hosted flow. You can only use the values `email` and `url` if you provide `relink_options`.
        """
        return_url: Optional[str]
        """
        The URL to redirect your customer back to after they link their accounts or cancel this Session. This parameter is required if `ui_mode` is `hosted`.
        """

    class Limits(StripeObject):
        accounts: int
        """
        The number of accounts that can be linked in this Session.
        """

    class ManualEntry(StripeObject):
        pass

    class RelinkOptions(StripeObject):
        account: Optional[str]
        """
        Requires the end user to repair this specific account during the authentication flow instead of connecting a different one.
        """
        authorization: str
        """
        The authorization to relink in the Session.
        """

    class RelinkResult(StripeObject):
        account: Optional[str]
        """
        The account relinked in the Session. Only present if `relink_options[account]` is set and relink is successful.
        """
        authorization: Optional[str]
        """
        The authorization relinked in the Session. Only present if relink is successful.
        """
        failure_reason: Optional[
            Literal["no_account", "no_authorization", "other"]
        ]
        """
        Reason for why relink failed. One of `no_authorization`, `no_account`, or `other`.
        """

    class StatusDetails(StripeObject):
        class Cancelled(StripeObject):
            reason: Literal["custom_manual_entry", "other"]
            """
            The reason for the Session being cancelled.
            """

        cancelled: Optional[Cancelled]
        _inner_class_types = {"cancelled": Cancelled}

    account_holder: Optional[AccountHolder]
    """
    The account holder for whom accounts are collected in this session.
    """
    accounts: ListObject["FinancialConnectionsAccountResource"]
    """
    The accounts that were collected as part of this Session.
    """
    client_secret: Optional[str]
    """
    A value that will be passed to the client to launch the authentication flow.
    """
    filters: Optional[Filters]
    hosted: Optional[Hosted]
    """
    Settings for the Hosted UI mode.
    """
    id: str
    """
    Unique identifier for the object.
    """
    limits: Optional[Limits]
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    manual_entry: Optional[ManualEntry]
    object: Literal["financial_connections.session"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    permissions: List[
        Literal["balances", "ownership", "payment_method", "transactions"]
    ]
    """
    Permissions requested for accounts collected during this session.
    """
    prefetch: Optional[
        List[
            Literal[
                "balances", "inferred_balances", "ownership", "transactions"
            ]
        ]
    ]
    """
    Data features requested to be retrieved upon account creation.
    """
    relink_options: Optional[RelinkOptions]
    relink_result: Optional[RelinkResult]
    return_url: Optional[str]
    """
    For webview integrations only. Upon completing OAuth login in the native browser, the user will be redirected to this URL to return to your app.
    """
    status: Optional[Literal["cancelled", "failed", "pending", "succeeded"]]
    """
    The current state of the session.
    """
    status_details: Optional[StatusDetails]
    ui_mode: Optional[Literal["hosted", "modal"]]
    """
    The UI mode for this session.
    """
    url: Optional[str]
    """
    The hosted URL for this Session. Redirect customers to this URL to take them to the hosted authentication flow. This value is only present when the Session is active and the `ui_mode` is `hosted`.
    """

    @classmethod
    def create(cls, **params: Unpack["SessionCreateParams"]) -> "Session":
        """
        To launch the Financial Connections authorization flow, create a Session. The session's client_secret can be used to launch the flow using Stripe.js.
        """
        return cast(
            "Session",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["SessionCreateParams"]
    ) -> "Session":
        """
        To launch the Financial Connections authorization flow, create a Session. The session's client_secret can be used to launch the flow using Stripe.js.
        """
        return cast(
            "Session",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["SessionRetrieveParams"]
    ) -> "Session":
        """
        Retrieves the details of a Financial Connections Session
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["SessionRetrieveParams"]
    ) -> "Session":
        """
        Retrieves the details of a Financial Connections Session
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "account_holder": AccountHolder,
        "filters": Filters,
        "hosted": Hosted,
        "limits": Limits,
        "manual_entry": ManualEntry,
        "relink_options": RelinkOptions,
        "relink_result": RelinkResult,
        "status_details": StatusDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.reporting._report_run import ReportRun as ReportRun
    from stripe.reporting._report_run_service import (
        ReportRunService as ReportRunService,
    )
    from stripe.reporting._report_type import ReportType as ReportType
    from stripe.reporting._report_type_service import (
        ReportTypeService as ReportTypeService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "ReportRun": ("stripe.reporting._report_run", False),
    "ReportRunService": ("stripe.reporting._report_run_service", False),
    "ReportType": ("stripe.reporting._report_type", False),
    "ReportTypeService": ("stripe.reporting._report_type_service", False),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
import json
from stripe._api_version import _ApiVersion
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from uuid import uuid4
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._funding_instructions import FundingInstructions
    from stripe._request_options import RequestOptions
    from stripe.params._customer_funding_instructions_create_params import (
        CustomerFundingInstructionsCreateParams,
    )


class CustomerFundingInstructionsService(StripeService):
    def create(
        self,
        customer: str,
        params: "CustomerFundingInstructionsCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FundingInstructions":
        """
        Retrieve funding instructions for a customer cash balance. If funding instructions do not yet exist for the customer, new
        funding instructions will be created. If funding instructions have already been created for a given customer, the same
        funding instructions will be retrieved. In other words, we will return the same funding instructions each time.
        """
        return cast(
            "FundingInstructions",
            self._request(
                "post",
                "/v1/customers/{customer}/funding_instructions".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        customer: str,
        params: "CustomerFundingInstructionsCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FundingInstructions":
        """
        Retrieve funding instructions for a customer cash balance. If funding instructions do not yet exist for the customer, new
        funding instructions will be created. If funding instructions have already been created for a given customer, the same
        funding instructions will be retrieved. In other words, we will return the same funding instructions each time.
        """
        return cast(
            "FundingInstructions",
            await self._request_async(
                "post",
                "/v1/customers/{customer}/funding_instructions".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def serialize_batch_create_funding_instructions(
        self,
        customer: str,
        params: Optional["CustomerFundingInstructionsCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a CustomerFundingInstructions create request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"customer": customer},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
import json
from stripe._api_version import _ApiVersion
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from uuid import uuid4
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.tax._transaction_create_from_calculation_params import (
        TransactionCreateFromCalculationParams,
    )
    from stripe.params.tax._transaction_create_reversal_params import (
        TransactionCreateReversalParams,
    )
    from stripe.params.tax._transaction_retrieve_params import (
        TransactionRetrieveParams,
    )
    from stripe.tax._transaction import Transaction
    from stripe.tax._transaction_line_item_service import (
        TransactionLineItemService,
    )

_subservices = {
    "line_items": [
        "stripe.tax._transaction_line_item_service",
        "TransactionLineItemService",
    ],
}


class TransactionService(StripeService):
    line_items: "TransactionLineItemService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def retrieve(
        self,
        transaction: str,
        params: Optional["TransactionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Retrieves a Tax Transaction object.
        """
        return cast(
            "Transaction",
            self._request(
                "get",
                "/v1/tax/transactions/{transaction}".format(
                    transaction=sanitize_id(transaction),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        transaction: str,
        params: Optional["TransactionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Retrieves a Tax Transaction object.
        """
        return cast(
            "Transaction",
            await self._request_async(
                "get",
                "/v1/tax/transactions/{transaction}".format(
                    transaction=sanitize_id(transaction),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create_from_calculation(
        self,
        params: "TransactionCreateFromCalculationParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Creates a Tax Transaction from a calculation, if that calculation hasn't expired. Calculations expire after 90 days.
        """
        return cast(
            "Transaction",
            self._request(
                "post",
                "/v1/tax/transactions/create_from_calculation",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_from_calculation_async(
        self,
        params: "TransactionCreateFromCalculationParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Creates a Tax Transaction from a calculation, if that calculation hasn't expired. Calculations expire after 90 days.
        """
        return cast(
            "Transaction",
            await self._request_async(
                "post",
                "/v1/tax/transactions/create_from_calculation",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create_reversal(
        self,
        params: "TransactionCreateReversalParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Partially or fully reverses a previously created Transaction.
        """
        return cast(
            "Transaction",
            self._request(
                "post",
                "/v1/tax/transactions/create_reversal",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_reversal_async(
        self,
        params: "TransactionCreateReversalParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Partially or fully reverses a previously created Transaction.
        """
        return cast(
            "Transaction",
            await self._request_async(
                "post",
                "/v1/tax/transactions/create_reversal",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def serialize_batch_create_reversal(
        self,
        params: Optional["TransactionCreateReversalParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Transaction create_reversal request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class PayoutIntent(StripeObject):
    """
    PayoutIntent represents an intent to send funds from a Financial Account to a payout method.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.payout_intent"]] = (
        "v2.money_management.payout_intent"
    )

    class From(StripeObject):
        currency: str
        """
        The currency of the financial account.
        """
        financial_account: str
        """
        The FinancialAccount that funds are pulled from.
        """

    class LatestPayout(StripeObject):
        outbound_payment: Optional[str]
        """
        The ID of the OutboundPayment, if applicable.
        """
        outbound_transfer: Optional[str]
        """
        The ID of the OutboundTransfer, if applicable.
        """
        type: Literal["outbound_payment", "outbound_transfer"]
        """
        The type of payout.
        """

    class NextAction(StripeObject):
        class HandleFailure(StripeObject):
            failure_reason: Literal[
                "account_not_configured_as_recipient",
                "currency_not_supported_for_financial_account_balance",
                "currency_required",
                "feature_not_active_for_recipient",
                "fx_rate_drift_exceeded_after_review",
                "insufficient_funds",
                "payout_method_account_type_incorrect",
                "payout_method_amount_limit_exceeded",
                "payout_method_canceled_by_customer",
                "payout_method_closed",
                "payout_method_currency_unsupported",
                "payout_method_declined",
                "payout_method_does_not_exist",
                "payout_method_expired",
                "payout_method_holder_address_incorrect",
                "payout_method_holder_details_incorrect",
                "payout_method_holder_name_incorrect",
                "payout_method_invalid_account_number",
                "payout_method_restricted",
                "payout_method_unsupported",
                "payout_method_usage_frequency_limit_exceeded",
                "recalled",
                "review_rejected",
                "to_destination_invalid",
                "unknown_failure",
            ]
            """
            Open Enum. The reason for the failure.
            """

        handle_failure: Optional[HandleFailure]
        """
        Details about a failure that requires user action. Populated when type is handle_failure.
        """
        type: Literal["handle_failure"]
        """
        Open Enum. The type of next action required.
        """
        _inner_class_types = {"handle_failure": HandleFailure}

    class RecipientNotification(StripeObject):
        setting: Literal["configured", "none"]
        """
        Closed Enum. Configuration option to enable or disable notifications to recipients.
        Do not send notifications when setting is NONE. Default to account setting when setting is CONFIGURED or not set.
        """

    class ScheduleOptions(StripeObject):
        execute_on: Optional[str]
        """
        The date when the payout should be executed, in YYYY-MM-DD format.
        """

    class StatusTransitions(StripeObject):
        canceled_at: Optional[str]
        """
        Timestamp describing when a PayoutIntent changed status to `canceled`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        posted_at: Optional[str]
        """
        Timestamp describing when a PayoutIntent changed status to `posted`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        processing_at: Optional[str]
        """
        Timestamp describing when a PayoutIntent changed status to `processing`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        requires_action_at: Optional[str]
        """
        Timestamp describing when a PayoutIntent changed status to `requires_action`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """

    class To(StripeObject):
        class PayoutMethodOptions(StripeObject):
            class BankAccount(StripeObject):
                class PreferredNetworkOptions(StripeObject):
                    class Ach(StripeObject):
                        submission: Optional[Literal["next_day", "same_day"]]
                        """
                        Open Enum. ACH submission timing.
                        """
                        transaction_purpose: Optional[Literal["payroll"]]
                        """
                        The transaction purpose for this ACH payment.
                        """

                    ach: Optional[Ach]
                    """
                    ACH-specific network options.
                    """
                    _inner_class_types = {"ach": Ach}

                preferred_network_options: Optional[PreferredNetworkOptions]
                """
                Per-network configuration options.
                """
                preferred_networks: List[
                    Literal[
                        "ach",
                        "becs",
                        "eft",
                        "fedwire",
                        "fps",
                        "npp",
                        "rtp",
                        "sepa_credit",
                        "sepa_instant",
                        "swift",
                    ]
                ]
                """
                The preferred networks to use for this PayoutIntent.
                """
                _inner_class_types = {
                    "preferred_network_options": PreferredNetworkOptions,
                }

            bank_account: Optional[BankAccount]
            """
            Options for bank account payout methods.
            """
            _inner_class_types = {"bank_account": BankAccount}

        currency: Optional[str]
        """
        The currency to send to the recipient.
        """
        payout_method: Optional[str]
        """
        The payout method ID. Optional for OutboundPayment if recipient has default payment method. Required for OutboundTransfer.
        """
        payout_method_options: Optional[PayoutMethodOptions]
        """
        Payout method options for the PayoutIntent.
        """
        recipient: Optional[str]
        """
        The recipient ID. Only relevant for OutboundPayment.
        """
        _inner_class_types = {"payout_method_options": PayoutMethodOptions}

    amount: Amount
    """
    The monetary amount to be sent.
    """
    created: str
    """
    Time at which the PayoutIntent was created.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    description: Optional[str]
    """
    An arbitrary string attached to the PayoutIntent. Often useful for displaying to users.
    """
    from_: From
    """
    The FinancialAccount that funds are pulled from.
    """
    id: str
    """
    Unique identifier for the PayoutIntent.
    """
    latest_payout: LatestPayout
    """
    Details about the latest payout associated with this PayoutIntent.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    next_action: Optional[NextAction]
    """
    Next action required for a PayoutIntent in the requires_action state. Populated when status is requires_action.
    """
    object: Literal["v2.money_management.payout_intent"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    recipient_notification: Optional[RecipientNotification]
    """
    Details about the OutboundPayment notification settings for recipient. Only applicable to OutboundPayment.
    """
    schedule_options: Optional[ScheduleOptions]
    """
    Scheduling options for the payout. If this is nil, we assume immediate execution.
    """
    statement_descriptor: Optional[str]
    """
    The description that appears on the receiving end for the payout (for example, on a bank statement).
    """
    status: Literal[
        "canceled", "pending", "posted", "processing", "requires_action"
    ]
    """
    Open Enum. Current status of the PayoutIntent: `pending`, `processing`, `posted`, `canceled`, `requires_action`.
    """
    status_transitions: Optional[StatusTransitions]
    """
    Hash containing timestamps of when transitioned to a particular status.
    """
    to: To
    """
    To which payout method the payout is sent.
    """
    _inner_class_types = {
        "from": From,
        "latest_payout": LatestPayout,
        "next_action": NextAction,
        "recipient_notification": RecipientNotification,
        "schedule_options": ScheduleOptions,
        "status_transitions": StatusTransitions,
        "to": To,
    }
    _field_remappings = {"from_": "from"}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._payment_method_configuration_create_params import (
        PaymentMethodConfigurationCreateParams,
    )
    from stripe.params._payment_method_configuration_list_params import (
        PaymentMethodConfigurationListParams,
    )
    from stripe.params._payment_method_configuration_modify_params import (
        PaymentMethodConfigurationModifyParams,
    )
    from stripe.params._payment_method_configuration_retrieve_params import (
        PaymentMethodConfigurationRetrieveParams,
    )


class PaymentMethodConfiguration(
    CreateableAPIResource["PaymentMethodConfiguration"],
    ListableAPIResource["PaymentMethodConfiguration"],
    UpdateableAPIResource["PaymentMethodConfiguration"],
):
    """
    PaymentMethodConfigurations control which payment methods are displayed to your customers when you don't explicitly specify payment method types. You can have multiple configurations with different sets of payment methods for different scenarios.

    There are two types of PaymentMethodConfigurations. Which is used depends on the [charge type](https://docs.stripe.com/connect/charges):

    **Direct** configurations apply to payments created on your account, including Connect destination charges, Connect separate charges and transfers, and payments not involving Connect.

    **Child** configurations apply to payments created on your connected accounts using direct charges, and charges with the on_behalf_of parameter.

    Child configurations have a `parent` that sets default values and controls which settings connected accounts may override. You can specify a parent ID at payment time, and Stripe will automatically resolve the connected account's associated child configuration. Parent configurations are [managed in the dashboard](https://dashboard.stripe.com/settings/payment_methods/connected_accounts) and are not available in this API.

    Related guides:
    - [Payment Method Configurations API](https://docs.stripe.com/connect/payment-method-configurations)
    - [Multiple configurations on dynamic payment methods](https://docs.stripe.com/payments/multiple-payment-method-configs)
    - [Multiple configurations for your Connect accounts](https://docs.stripe.com/connect/multiple-payment-method-configurations)
    """

    OBJECT_NAME: ClassVar[Literal["payment_method_configuration"]] = (
        "payment_method_configuration"
    )

    class AcssDebit(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Affirm(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class AfterpayClearpay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Alipay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Alma(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class AmazonPay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class ApplePay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class AuBecsDebit(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class BacsDebit(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Bancontact(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Billie(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Bizum(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Blik(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Boleto(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Card(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class CartesBancaires(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Cashapp(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Crypto(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class CustomerBalance(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Eps(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Fpx(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Giropay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class GooglePay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Gopay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Grabpay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class IdBankTransfer(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Ideal(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Jcb(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class KakaoPay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Klarna(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Konbini(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class KrCard(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Link(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class MbWay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Mobilepay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Multibanco(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class NaverPay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class NzBankAccount(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Oxxo(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class P24(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class PayByBank(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Payco(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Paynow(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Paypal(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Paypay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Payto(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Pix(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Promptpay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Qris(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class RevolutPay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class SamsungPay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Satispay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Scalapay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class SepaDebit(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Shopeepay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Sofort(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Sunbit(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Swish(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Twint(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Upi(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class UsBankAccount(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class WechatPay(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    class Zip(StripeObject):
        class DisplayPreference(StripeObject):
            overridable: Optional[bool]
            """
            For child configs, whether or not the account's preference will be observed. If `false`, the parent configuration's default is used.
            """
            preference: Literal["none", "off", "on"]
            """
            The account's display preference.
            """
            value: Literal["off", "on"]
            """
            The effective display preference value.
            """

        available: bool
        """
        Whether this payment method may be offered at checkout. True if `display_preference` is `on` and the payment method's capability is active.
        """
        display_preference: DisplayPreference
        _inner_class_types = {"display_preference": DisplayPreference}

    acss_debit: Optional[AcssDebit]
    active: bool
    """
    Whether the configuration can be used for new payments.
    """
    affirm: Optional[Affirm]
    afterpay_clearpay: Optional[AfterpayClearpay]
    alipay: Optional[Alipay]
    alma: Optional[Alma]
    amazon_pay: Optional[AmazonPay]
    apple_pay: Optional[ApplePay]
    application: Optional[str]
    """
    For child configs, the Connect application associated with the configuration.
    """
    au_becs_debit: Optional[AuBecsDebit]
    bacs_debit: Optional[BacsDebit]
    bancontact: Optional[Bancontact]
    billie: Optional[Billie]
    bizum: Optional[Bizum]
    blik: Optional[Blik]
    boleto: Optional[Boleto]
    card: Optional[Card]
    cartes_bancaires: Optional[CartesBancaires]
    cashapp: Optional[Cashapp]
    crypto: Optional[Crypto]
    customer_balance: Optional[CustomerBalance]
    eps: Optional[Eps]
    fpx: Optional[Fpx]
    giropay: Optional[Giropay]
    google_pay: Optional[GooglePay]
    gopay: Optional[Gopay]
    grabpay: Optional[Grabpay]
    id: str
    """
    Unique identifier for the object.
    """
    id_bank_transfer: Optional[IdBankTransfer]
    ideal: Optional[Ideal]
    is_default: bool
    """
    The default configuration is used whenever a payment method configuration is not specified.
    """
    jcb: Optional[Jcb]
    kakao_pay: Optional[KakaoPay]
    klarna: Optional[Klarna]
    konbini: Optional[Konbini]
    kr_card: Optional[KrCard]
    link: Optional[Link]
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    mb_way: Optional[MbWay]
    mobilepay: Optional[Mobilepay]
    multibanco: Optional[Multibanco]
    name: str
    """
    The configuration's name.
    """
    naver_pay: Optional[NaverPay]
    nz_bank_account: Optional[NzBankAccount]
    object: Literal["payment_method_configuration"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    oxxo: Optional[Oxxo]
    p24: Optional[P24]
    parent: Optional[str]
    """
    For child configs, the configuration's parent configuration.
    """
    pay_by_bank: Optional[PayByBank]
    payco: Optional[Payco]
    paynow: Optional[Paynow]
    paypal: Optional[Paypal]
    paypay: Optional[Paypay]
    payto: Optional[Payto]
    pix: Optional[Pix]
    promptpay: Optional[Promptpay]
    qris: Optional[Qris]
    revolut_pay: Optional[RevolutPay]
    samsung_pay: Optional[SamsungPay]
    satispay: Optional[Satispay]
    scalapay: Optional[Scalapay]
    sepa_debit: Optional[SepaDebit]
    shopeepay: Optional[Shopeepay]
    sofort: Optional[Sofort]
    sunbit: Optional[Sunbit]
    swish: Optional[Swish]
    twint: Optional[Twint]
    upi: Optional[Upi]
    us_bank_account: Optional[UsBankAccount]
    wechat_pay: Optional[WechatPay]
    zip: Optional[Zip]

    @classmethod
    def create(
        cls, **params: Unpack["PaymentMethodConfigurationCreateParams"]
    ) -> "PaymentMethodConfiguration":
        """
        Creates a payment method configuration
        """
        return cast(
            "PaymentMethodConfiguration",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["PaymentMethodConfigurationCreateParams"]
    ) -> "PaymentMethodConfiguration":
        """
        Creates a payment method configuration
        """
        return cast(
            "PaymentMethodConfiguration",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["PaymentMethodConfigurationListParams"]
    ) -> ListObject["PaymentMethodConfiguration"]:
        """
        List payment method configurations
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["PaymentMethodConfigurationListParams"]
    ) -> ListObject["PaymentMethodConfiguration"]:
        """
        List payment method configurations
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls,
        id: str,
        **params: Unpack["PaymentMethodConfigurationModifyParams"],
    ) -> "PaymentMethodConfiguration":
        """
        Update payment method configuration
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "PaymentMethodConfiguration",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls,
        id: str,
        **params: Unpack["PaymentMethodConfigurationModifyParams"],
    ) -> "PaymentMethodConfiguration":
        """
        Update payment method configuration
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "PaymentMethodConfiguration",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls,
        id: str,
        **params: Unpack["PaymentMethodConfigurationRetrieveParams"],
    ) -> "PaymentMethodConfiguration":
        """
        Retrieve payment method configuration
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls,
        id: str,
        **params: Unpack["PaymentMethodConfigurationRetrieveParams"],
    ) -> "PaymentMethodConfiguration":
        """
        Retrieve payment method configuration
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "acss_debit": AcssDebit,
        "affirm": Affirm,
        "afterpay_clearpay": AfterpayClearpay,
        "alipay": Alipay,
        "alma": Alma,
        "amazon_pay": AmazonPay,
        "apple_pay": ApplePay,
        "au_becs_debit": AuBecsDebit,
        "bacs_debit": BacsDebit,
        "bancontact": Bancontact,
        "billie": Billie,
        "bizum": Bizum,
        "blik": Blik,
        "boleto": Boleto,
        "card": Card,
        "cartes_bancaires": CartesBancaires,
        "cashapp": Cashapp,
        "crypto": Crypto,
        "customer_balance": CustomerBalance,
        "eps": Eps,
        "fpx": Fpx,
        "giropay": Giropay,
        "google_pay": GooglePay,
        "gopay": Gopay,
        "grabpay": Grabpay,
        "id_bank_transfer": IdBankTransfer,
        "ideal": Ideal,
        "jcb": Jcb,
        "kakao_pay": KakaoPay,
        "klarna": Klarna,
        "konbini": Konbini,
        "kr_card": KrCard,
        "link": Link,
        "mb_way": MbWay,
        "mobilepay": Mobilepay,
        "multibanco": Multibanco,
        "naver_pay": NaverPay,
        "nz_bank_account": NzBankAccount,
        "oxxo": Oxxo,
        "p24": P24,
        "pay_by_bank": PayByBank,
        "payco": Payco,
        "paynow": Paynow,
        "paypal": Paypal,
        "paypay": Paypay,
        "payto": Payto,
        "pix": Pix,
        "promptpay": Promptpay,
        "qris": Qris,
        "revolut_pay": RevolutPay,
        "samsung_pay": SamsungPay,
        "satispay": Satispay,
        "scalapay": Scalapay,
        "sepa_debit": SepaDebit,
        "shopeepay": Shopeepay,
        "sofort": Sofort,
        "sunbit": Sunbit,
        "swish": Swish,
        "twint": Twint,
        "upi": Upi,
        "us_bank_account": UsBankAccount,
        "wechat_pay": WechatPay,
        "zip": Zip,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired, TypedDict


class GiftCardActivateParams(RequestOptions):
    balance: NotRequired["GiftCardActivateParamsBalance"]
    """
    The initial balance to set on the gift card.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    on_behalf_of: NotRequired[str]
    """
    The Stripe account ID to process the gift card operation on behalf of.
    """


class GiftCardActivateParamsBalance(TypedDict):
    amount: int
    """
    The initial balance amount to be loaded when activating the gift card, in the smallest currency unit
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
